#define bool int
#define false 0
#define true 1

typedef struct {
        unsigned short  type;
        unsigned short len;
        unsigned int value;
} NMRP_OPT;

typedef struct {
        unsigned short reserved;
        unsigned char code;
        unsigned char id;
        unsigned short length;

        NMRP_OPT opt;
} NMRP_MSG;


#define ETHER_NMRP              0x0912

#define NMRP_HDR_LEN            6//(sizeof(NMRP_MSG) - sizeof(NMRP_OPT)-2)
#define NMRP_MIN_OPT_LEN        4 //(sizeof(NMRP_OPT) - 1)

#define NMRP_MAGIC_NO   "\x4E\x54\x47\x52"
#define NMRP_MAGIC_NO_LEN       (sizeof(NMRP_MAGIC_NO) - 1)


/* NMRP codes */
enum _nmrp_codes_ {
        NMRP_CODE_ADVERTISE             = 0x01,
        NMRP_CODE_CONF_REQ              = 0x02,
        NMRP_CODE_CONF_ACK              = 0x03,
        NMRP_CODE_CLOSE_REQ             = 0x04,
        NMRP_CODE_CLOSE_ACK             = 0x05,
        NMRP_CODE_TFTP_UL_REQ   = 0x10
};

/* NMRP option types */
enum _nmrp_option_types_ {
        NMRP_OPT_MAGIC_NO       = 0x0001,
        NMRP_OPT_DEV_IP         = 0x0002,
        NMRP_OPT_FW_UP          = 0x0101
};
/* NMRP REQ max retries */
enum _nmrp_req_max_retries_ {
        NMRP_MAX_RETRY_CONF             = 5,
        NMRP_MAX_RETRY_CLOSE    = 2,
        NMRP_MAX_RETRY_TFTP_UL  = 4
};

/* NMRP timeouts */
enum _nmrp_timeouts_ {
        NMRP_TIMEOUT_REQ                =   500, /* 0.5 sec */
        NMRP_TIMEOUT_LISTEN             =  3000, /* 3 sec */
        NMRP_TIMEOUT_ACTIVE             = 60000, /* 1 minute */
        NMRP_TIMEOUT_CLOSE              =  6000, /* 6 sec */
        NMRP_TIMEOUT_ADVERTISE  =   500  /* 0.5 sec */
};

/*============== (1) Ptorocol related definitions (End) ===============*/


/*============== (2) Implementation related definitions ===============*/

#define NMRP_MAX_OPT_PER_MSG    (6)


enum _nmrp_errors_ {
        NMRP_ERR_NONE = 0,
        NMRP_ERR_MSG_INVALID_LEN,
        NMRP_ERR_MSG_TOO_LONG,
        NMRP_ERR_MSG_TOO_MANY_OPT,
        NMRP_ERR_MSG_UNKNOWN_OPT,
        NMRP_ERR_MSG_INVALID_OPT,
        NMRP_ERR_NO_BUF,
        NMRP_ERR_GENERAL
};

typedef struct {
        unsigned short type;
        unsigned short len;

        union {
                unsigned char byteVal;
                unsigned short  wordVal;
                unsigned int dwordVal;
                char *streamVal;
        } value;
} NMRP_PARSED_OPT;
typedef struct {
        unsigned char code;
        unsigned char id;
        unsigned short length;

        int numOptions;
        NMRP_PARSED_OPT options[NMRP_MAX_OPT_PER_MSG];
} NMRP_PARSED_MSG;

typedef unsigned char enet_addr_t[6];
typedef unsigned char ip_addr_t[4];

/*
 * Ethernet header.
 */
typedef struct {
    enet_addr_t   destination;
    enet_addr_t   source;
    unsigned short          type;
#define ETH_TYPE_IP   0x800
#define ETH_TYPE_ARP  0x806
#define ETH_TYPE_RARP 0x8053
} eth_header_t;

/*
 * A IP<->ethernet address mapping.
 */
typedef struct {
    ip_addr_t    ip_addr;
    enet_addr_t  enet_addr;
} ip_route_t;

int NMRP_MsgParsing(unsigned char *pkt, int pktLen, NMRP_PARSED_MSG *msg);
NMRP_PARSED_OPT *NMRP_MsgGetOpt(NMRP_PARSED_MSG *msg, int optType);


