/*
 * Tiny Embedded JavaScript parser
 *
 * Copyright 2004, Broadcom Corporation
 * All Rights Reserved.
 * 
 * THIS SOFTWARE IS OFFERED "AS IS", AND BROADCOM GRANTS NO WARRANTIES OF ANY
 * KIND, EXPRESS OR IMPLIED, BY STATUTE, COMMUNICATION OR OTHERWISE. BROADCOM
 * SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A SPECIFIC PURPOSE OR NONINFRINGEMENT CONCERNING THIS SOFTWARE.
 *
 * $Id: basic.c,v 1.3 2007/04/30 08:19:54 lloyd Exp $
 */

#include <stdarg.h>
#include "boa.h"
#include "httpd.h"
#include <signal.h>  
#include <string.h>

#include <sys/klog.h>
#include <sys/sysinfo.h>
#include <stdio.h>


#ifdef CONFIG_USER_INT6000           /* this define is determined by Makfile */
#include "int6kwebapi.h"
#endif
#ifdef CONFIG_USER_VOIP_ATA			 /* this define is determined by Makfile */
#include "mini_ata.h"
#endif


/* Copy each token in wordlist delimited by space into word */
#define foreach(word, wordlist, next) \
	for (next = &wordlist[strspn(wordlist, " ")], \
	     strncpy(word, next, sizeof(word)), \
	     word[strcspn(word, " ")] = '\0', \
	     word[sizeof(word) - 1] = '\0', \
	     next = strchr(next, ' '); \
	     strlen(word); \
	     next = next ? &next[strspn(next, " ")] : "", \
	     strncpy(word, next, sizeof(word)), \
	     word[strcspn(word, " ")] = '\0', \
	     word[sizeof(word) - 1] = '\0', \
	     next = strchr(next, ' '))
	     
extern char *nvram_get(char *name);
static int so_printf(webs_t r, const char *fmt, ...);	     

#define MAX_STRING_LEN	1024

#define DPRINTF		

/* Look for unquoted character within a string */
static char *
unqstrstr(char *haystack, char *needle)
{
	char *cur;
	int q;

	for (cur = haystack, q = 0;
	     cur < &haystack[strlen(haystack)] && !(!q && !strncmp(needle, cur, strlen(needle)));
	     cur++) {
		if (*cur == '"')
			q ? q-- : q++;
	}
	return (cur < &haystack[strlen(haystack)]) ? cur : NULL;
}

static char *
get_arg(char *args, char **next)
{
	char *arg, *end;

	/* Parse out arg, ... */
	if (!(end = unqstrstr(args, ","))) {
		end = args + strlen(args);
		*next = NULL;
	} else
		*next = end + 1;

	/* Skip whitespace and quotation marks on either end of arg */
	for (arg = args; isspace((int)*arg) || *arg == '"'; arg++);
	for (*end-- = '\0'; isspace((int)*end) || *end == '"'; end--)
		*end = '\0';

	return arg;
}

static void
call(char *func, webs_t	stream)
{
	char *args, *end, *next;
	int argc;
	char * argv[16];
	struct ej_handler *handler;

	/* Parse out ( args ) */
	if (!(args = strchr(func, '(')))
		return;
	if (!(end = unqstrstr(func, ")")))
		return;
	*args++ = *end = '\0';

	/* Set up argv list */
	for (argc = 0; argc < 16 && args && *args; argc++, args = next) {
		if (!(argv[argc] = get_arg(args, &next)))
			break;
	}

	/* Call handler */
	for (handler = &ej_handlers[0]; handler->pattern; handler++) {
		if (strncmp(handler->pattern, func, strlen(handler->pattern)) == 0)
			handler->output(0, stream, argc, argv);
	}
}

int so_printf(webs_t r, const char *fmt, ...)
{
	char buffer[MAX_STRING_LEN+50];
	va_list argptr;
	int nbytes;

	va_start(argptr, fmt);
	nbytes = vsprintf(buffer, fmt, argptr);
	va_end(argptr);
	
	return req_write(r, buffer);
}	


static int
ej_get_firmware_ver(int eid, webs_t req, int argc, char **argv)
{
	char szVer[15];
	
	return websWrite(req, "%s", "1.00.00");
}


/*
 * Example: 
 * wan_proto=dhcp
 * <% nvram_invmatch("wan_proto", "dhcp", "disabled"); %> does not produce
 * <% nvram_invmatch("wan_proto", "static", "disabled"); %> produces "disabled"
 */
static int
ej_nvram_invmatch(int eid, webs_t wp, int argc, char **argv)
{
	char *name, *invmatch, *output;

	if (ejArgs(argc, argv, "%s %s %s", &name, &invmatch, &output) < 3) {
		//websError(wp, 400, "Insufficient args\n");
		return -1;
	}

	if (nvram_invmatch(name, invmatch))
		return websWrite(wp, output);

	return 0;
}	

/*
 * Example: 
 * filter_maclist=00:12:34:56:78:00 00:87:65:43:21:00
 * <% nvram_list("filter_maclist", 1); %> produces "00:87:65:43:21:00"
 * <% nvram_list("filter_maclist", 100); %> produces ""
 */
static int
ej_nvram_list(int eid, webs_t wp, int argc, char **argv)
{
	char *name;
	int which;
	char word[256], *next;

	if (ejArgs(argc, argv, "%s %d", &name, &which) < 2) {
		//websError(wp, 400, "Insufficient args\n");
		return -1;
	}

	foreach(word, nvram_safe_get(name), next) {
		if (which-- == 0)
			return websWrite(wp, word);
	}

	return 0;
}

#if !defined(NVRAM_SHARED_LIB)
/* have to move to shared library */
/* these nvram related function for temperate testing only */
static char *nvram_get(char *name)
{
	
	char nvram[100] = "";
	
	if (strcmp(name, "belkin_router") == 0)
	  strcpy(nvram, "1");
	else if (strcmp(name, "wan_ifname") == 0)
	  strcpy(nvram, "eth1");
	
	return nvram;  
}	    

int nvram_match(char *name, char *match)
{
	char *c;
	int ret = 0;
	
	c = nvram_get(name);
	if (c != NULL) {
		if (strcmp(c, match) == 0)
		  ret = 1;
	}
	return ret;
}

int nvram_invmatch(char *name, char *invmatch)
{
	char *c;
	int ret = 1;
	
	c = nvram_get(name);
	if (c != NULL) {
		if (strcmp(c, invmatch) == 0)
		  ret = 0;
	}
	return ret;
}		  
/* these nvram related function for temperate testing only */
#endif

/*
 * Example: 
 * lan_ipaddr=192.168.1.1
 * <% nvram_get("lan_ipaddr"); %> produces "192.168.1.1"
 * <% nvram_get("undefined"); %> produces ""
 */
static int
ej_nvram_get(int eid, webs_t wp, int argc, char **argv)
{
	char *name, *c;
	int ret = 0;

	if (ejArgs(argc, argv, "%s", &name) < 1) {
		//websError(wp, 400, "Insufficient args\n");
		return -1;
	}

	for (c = nvram_safe_get(name); *c; c++) {
		if (isprint((int) *c) &&
		    *c != '"' && *c != '&' && *c != '<' && *c != '>') {
			ret += websWrite(wp, "%c", *c);
		}	
		else
			ret += websWrite(wp, "&#%d", *c);
	}

	return ret;
}

/*
 * Example: 
 * wan_proto=dhcp
 * <% nvram_match("wan_proto", "dhcp", "selected"); %> produces "selected"
 * <% nvram_match("wan_proto", "static", "selected"); %> does not produce
 */
static int
ej_nvram_match(int eid, webs_t wp, int argc, char **argv)
{
	char *name, *match, *output;

	if (ejArgs(argc, argv, "%s %s %s", &name, &match, &output) < 3) {
		//websError(wp, 400, "Insufficient args\n");
		return -1;
	}

	if (nvram_match(name, match))
		return websWrite(wp, output);

	return 0;
}	




static int
ej_value_match(int eid, webs_t wp, int argc, char **argv)
{
	char *name, *match, *output, *type;
	char value[30];

	memset(value, 0, sizeof(value));

	if (ejArgs(argc, argv, "%s %s %s %s", &name, &match, &output, &type) < 3) {
		//websError(wp, 400, "Insufficient args\n");
		return -1;
	}
#ifdef CONFIG_USER_VOIP_ATA
	if (strncmp(type, "voip", strlen("voip")) == 0){
		voip_parm_get(name, value);
	}
#endif
    if (strncmp(value, match, strlen(match)) == 0){
		return websWrite(wp, output);
	}

	return 0;
}	




static int
ej_wanstatus_get(int eid, webs_t wp, int argc, char **argv)
{
	/* up -> connected
	   down -> disconnected */
	   
	//return websWrite(wp, "Up");
	
	int ifno;
	char buf[20];
	char *pWanIp =(char *)nvram_get("wan0_ipaddr");

	if (ejArgs(argc, argv,  "%d", &ifno) < 1) {
		//websError(wp, 400, "Insufficient args\n");
		return -1;
	}
	
	
	if (ifno == 0) /* default interface */
	  ifno = atoi(nvram_safe_get("wan_default_iface"));  		
	else if(ifno > 0) 
		ifno--;	/* 1-> wan0, 2->wan1, 3->wan2, 4->wan3 */
	sprintf(buf, "wan%d_ipaddr", ifno);
	pWanIp = (char *)nvram_get(buf);
	
    	DPRINTF("%s:: wan ip =%s\n",__FUNCTION__, pWanIp);
    
    	if (pWanIp && (inet_addr(pWanIp))) {
		return websWrite(wp, "Up");
	}	
    	else {
        	return websWrite(wp, "Down");
        }	
		
}


static char *
rfctime(const time_t *timep)
{
	static char s[201];
	struct tm tm;

	memcpy(&tm, localtime(timep), sizeof(struct tm));
//	strftime(s, 200, "%a, %d %b %Y %H:%M:%S %z", &tm);
	strftime(s, 200, "%Y/%m/%d,%H:%M:%S", &tm);
	return s;
}
/* Report time in RFC-822 format */
static int
ej_localtime(int eid, webs_t wp, int argc, char **argv)
{
	time_t tm;
    
	time(&tm);
//	printf("current time is %s\n", ctime(&tm));
	
	return websWrite(wp, rfctime(&tm));
}

/*lease_t defined the same as struct dhcpOfferedAddr in 
busybox/networking/udhcp/leases.h*/
#define        HNAME_SIZE              26
struct lease_t {
	uint8_t chaddr[16];
    uint32_t yiaddr;	/* network order */
	uint32_t expires;	/* host order */
    char	hostname[32];
	/* uint8_t hostname[HNAME_SIZE];   Add by jku: hostname */
    /*  uint8_t type;          jku: static, dynamic, reserve */
};


long GetSysUpTime()
{
	struct sysinfo info;

	sysinfo(&info);
	return info.uptime;
}

/*
 * Reads file and returns contents
 * @param	fd	file descriptor
 * @return	contents of file or NULL if an error occurred
 */
char *
fd2str(int fd)
{
	char *buf = NULL;
	size_t count = 0, n;

	do {
		buf = realloc(buf, count + 512);
		n = read(fd, buf + count, 512);
		if (n < 0) {
			free(buf);
			buf = NULL;
		}
		count += n;
	} while (n == 512);

	close(fd);
	if (buf)
		buf[count] = '\0';
	return buf;
}

/*
 * Reads file and returns contents
 * @param	path	path to file
 * @return	contents of file or NULL if an error occurred
 */
char *
file2str(const char *path)
{
	int fd;

	if ((fd = open(path, O_RDONLY)) == -1) {
		perror(path);
		return NULL;
	}

	return fd2str(fd);
}


/**********************************
return value:
	0:	fail to open var/run/XXX.pid file
	pid: pid number
***********************************/
int get_task_pid(char *strTask)
{
	int pid; 
	char filename[80];
	char *strPid;

	
	snprintf(filename, sizeof(filename), "/var/run/%s.pid", strTask);
	if( (strPid = file2str( filename )) )
	{
		pid = atoi(strPid);
		free( strPid );
	}
	else
		pid = 0;;

	return pid;
}


/**********************************
return value:
	0:	task of this pid is dead
	1: 	task of this pid is alive
***********************************/
int pid_is_running(int pid)
{
	char filename[80];
	FILE *fp;
	
	snprintf(filename, sizeof(filename), "/proc/%d/status", pid);

	if((fp = fopen(filename, "r"))){
		//fgets(line, sizeof(line), fp);
        	/* Buffer should contain a string like "Name:   binary_name" */
		//sscanf(line, "%*s %s", name);
        	fclose(fp);
		return 1;
        }
	
	 //cprintf("cannot open %s\n", filename);
        return 0;
}

/*********************************************************************
return value:
	0:	task is dead
	1:	task is alive
	2:	cannot open XXX.pid file(this task is never brought up from system start)
*********************************************************************/
int task_is_running(char *strTask)
{
	int pid;
	
	if( (pid=get_task_pid(strTask)) )
	{
		return pid_is_running(pid);
	}
	else
	{
		//cprintf("fail to open var/run/%s.pid\n", strTask);
		return 2;
	}
}


/* Always lead with "Firewall: " */
int firewall_log(webs_t wp, char *ptr, FILE *fp)
{
    char *tok1, *tok2;
    int ret;
    
    if (strstr(ptr, "Firewall:")) {
    	char buf[256];

    	memset(buf, 0, sizeof(buf));
    	strcpy(buf, ptr);
    	strtok(buf, ":");
    	strtok(NULL, ":");
		tok1 = strtok(NULL, ":");
		tok2 = strtok(NULL, "\n");
		ret += websWrite(wp, "[DoS attack: %s] %s", tok1, tok2);	
		ret += websWrite(wp, "<br>" );	
	}
	//return 0;				 		
	return ret;
}	
	
#if porting20081017
void
clear_sysLog(void)
{
    char cmd[50], *strPid;
    int logProc;

    if(strPid =file2str( PROC_PID_FILE )){
        logProc = atoi(strPid);
        free(strPid);
        sprintf(cmd,"kill -11 %d",logProc);
        system(cmd);
    }
} 
#endif
#define BUFSIZE		8196
int dump_firewall_log(webs_t wp) 
{
	char *buf=NULL;
	int cmd=3;	/* 4 -> read/clear */
	int lastc;
	int i, n;	
	int out_cnt = 0;
	char temp[300];	
	FILE *fp;
	
	buf = malloc(BUFSIZE);	
	if ((n = klogctl(cmd, buf, BUFSIZE)) < 0) {
		if (buf)
			free(buf);
		return 0;
	}	
#if 0	
	lastc = '\n';
	for (i = 0; i < n; i++) {
		if (lastc == '\n' && buf[i] == '<') {
			i++;
			while (buf[i] >= '0' && buf[i] <= '9')
				i++;
			if (buf[i] == '>')
				i++;
		}
		lastc = buf[i];
		websWrite(wp, "%c", lastc);
		out_cnt++;
	}
	if (lastc != '\n') {
		websWrite(wp, '\n');
		out_cnt++;
	}	
all_done:
	if (buf) {
		free(buf);
	}
	
	return out_cnt;
#else
	
#if 0
	temp[0] = '\0';
	out_cnt = 0;
 	fp = fopen("/var/log/firewalllog.txt", "w+");
	for (i = 0; i < n; i++) {
		temp[out_cnt++] = buf[i];
		if (buf[i] == '\n') {
			temp[out_cnt] = '\0';
			firewall_log(wp, temp, fp);
			out_cnt = 0;
		}	
		if (out_cnt >= (sizeof(temp)-1)) {
			temp[out_cnt] = '\0';
			firewall_log(wp, temp, fp);
			out_cnt = 0;			
		}	
	}
	if (out_cnt != 0) {
		temp[out_cnt] = '\0';
		firewall_log(wp, temp, fp);		
	}	
	fclose(fp);
	return n;								
#else
{
    char line[250];
    
    n=0;
    fp = fopen("/var/log/firewalllog.txt", "r");
    while( fgets(line, sizeof(line), fp)!=NULL)
	{
		n += firewall_log(wp, line, fp);
	}
	fclose(fp);
	return n;
}
#endif
	
#endif	

}	

#if porting20081017
static int
ej_dumpSysLog(int eid, webs_t wp, int argc, char_t **argv)
{
    int  ret = 0;
    FILE *fp;
    char content[200];
    char cmd[50], *strPid;
    int logProc;

    if(strPid =file2str( PROC_PID_FILE )){
        logProc = atoi(strPid);
        free(strPid);
        sprintf(cmd,"kill -10 %d",logProc);
        system(cmd);
        ret += websWrite(wp, "System Log...\n" );
        ret += websWrite(wp, "<br>" );

        if ((fp = fopen(LOG_FILE_PATH,"r"))!=NULL){
            while (fgets(content, 200, fp) != NULL)
            {
                ret += websWrite(wp, "%s",content);
                ret += websWrite(wp, "<br>");
            }
            fclose(fp);
        }
        
        /* add start, vicky, 2008-5-19 */
    	if( strcmp(nvram_get("fw_disable"), "0") == 0 )
    	{
    	    ret += websWrite(wp, "Firewall Log...\n" );
	        ret += websWrite(wp, "<br>" );   
	        ret += dump_firewall_log(wp);
        }
        /* add end, vicky, 2008-5-19 */        
    }
    return ret;
}
#endif

static int
ej_dumplog(int eid, webs_t wp, int argc, char_t **argv)
{
	int  ret = 0;
	FILE *fp;
	char content[200];

	//Brian+: dump system log start
	
    	ret += websWrite(wp, "System Log...\n" );
	ret += websWrite(wp, "<br>" );
	
	if ((fp = fopen("/var/log/messages","r"))!=NULL)

	{
		while (fgets(content, 200, fp) != NULL)
		{	
			ret += websWrite(wp, "%s",content);	
			ret += websWrite(wp, "<br>");	
		}
		fclose(fp);
	}
    	ret += websWrite(wp, "Firewall Log...\n" );
	ret += websWrite(wp, "<br>" );   
	
	ret += dump_firewall_log(wp);
	
	return ret;	 		
}

#if porting20081017
static int
ej_dumpleases(int eid, webs_t wp, int argc, char_t **argv) 
{
	FILE *fp;
	struct lease_t lease;
	int i;
	struct in_addr addr;
	unsigned long expires;
	char sigusr1[] = "-XX";
	int ret = 0, arp_ret = 0;
	int count = 0;
	char *ipaddr, my_ip[20]="", my_mac[20]="", my_lanif[10]="", mac[20]="", expires_time[50]="";
	int pid;

#if 0
        /* Write out leases file */
        sprintf(sigusr1, "-%d", SIGUSR1);
        eval("killall", sigusr1, "udhcpd");
#endif    

	/* Write out leases file */
	pid = get_task_pid("udhcpd");
	if (pid != 0) {	   
		kill(pid, SIGUSR1);    
		sleep(1);		/* waiting writed completed */
	}	

	/* Parse leases file */
	if ((fp = fopen("/var/tmp/udhcpd.leases", "r"))) {   
	    while (fread(&lease, sizeof(lease), 1, fp)) {   
		strcpy(mac,"");

		for (i = 0; i < 6; i++) {    
			sprintf(mac+strlen(mac),"%02X", lease.chaddr[i]);  
			if (i != 5) sprintf(mac+strlen(mac),":");
		}    
		mac[17] = '\0';
		if(!strcmp(mac,"00:00:00:00:00:00"))
			continue;
		
		addr.s_addr = lease.yiaddr;     

        //checking client is on-line or not by ARP
        strcpy(my_mac,nvram_get("lan_hwaddr"));
        strcpy(my_ip,nvram_get("lan_ipaddr"));
        strcpy(my_lanif,nvram_get("lan_ifname"));

        arp_ret = arpping(addr.s_addr, inet_addr(my_ip), my_mac, my_lanif);
        if (arp_ret) { 
            continue;   //no arp response
        }

		ipaddr = inet_ntoa(addr);
		expires = ntohl(lease.expires);       
		strcpy(expires_time,"");
	

        if (!expires){
 			continue;
			strcpy(expires_time,"expired");
		}
		else {
			if (expires > 60*60*24) {
				sprintf(expires_time+strlen(expires_time),"%ld days, ",expires / (60*60*24));
				expires %= 60*60*24;
			}
			if (expires > 60*60) {
				sprintf(expires_time+strlen(expires_time),"%02ld:",expires / (60*60));	// hours
				expires %= 60*60;
			}
			else{
				sprintf(expires_time+strlen(expires_time),"00:");	// no hours
			}
			if (expires > 60) {
				sprintf(expires_time+strlen(expires_time),"%02ld:",expires / 60);	// minutes
				expires %= 60;
			}
			else{
				sprintf(expires_time+strlen(expires_time),"00:");	// no minutes
			}

			sprintf(expires_time+strlen(expires_time),"%02ld:",expires);		// seconds

			expires_time[strlen(expires_time)-1]='\0';
		     }   			/*!*lease.hostname ? "&nbsp;" : lease.hostname, */
		ret += websWrite(wp, "%c\"%s\",\"%s\",\"%s\"", //Brian+ for change show order
				count ? ',' :' ',
				mac,  
				lease.hostname,
				ipaddr
				);
		count++;
		}
	fclose(fp);
	}

	return ret;
}
#endif

#if porting20081017
static int
ej_login_ipaddr(int eid, webs_t wp, int argc, char **argv)
{
	return websWrite(wp, "%s", client_req.client_ip);	
}
#endif

static int
ej_restart_delay_get(int eid, webs_t wp, int argc, char **argv)
{
	return websWrite(wp, "%d", 25);	
}

//CL
char * 
get_http_client_hwaddr(char *ip)
{
	FILE *fp;
        char line[100];

	char ipa[50];           // ip address
	char hwa[50];           // HW address 
	char mask[50];          // ntemask   
	char dev[50];           // interface
	int type;               // HW type
	int flags;              // flags
	static char mac[20]; 
	memset(mac,0,20);						

        if ((fp = fopen("/proc/net/arp", "r")) == NULL) 
			return NULL;
        	
        
	      // Read the ARP cache entries.
	      // IP address       HW type     Flags       HW address            Mask     Device
	      // 192.168.2.1      0x1         0x2         xx:xx:xx:xx:xx:xx     *          br0
		while( fgets(line, sizeof(line), fp)!=NULL)
		{
			if(sscanf(line, "%s 0x%x 0x%x %s %s %s\n", ipa, &type, &flags, hwa, mask, dev)!=6)
				continue;
			
			if(strcmp(ip, ipa))
				continue;
			
				strcpy(mac, hwa);
				break;					
		}
		/* for check MAC not found, default WAN mac address */
		if (strcmp("br1",dev))
		    strcpy(mac,nvram_get("wan0_hwaddr"));

	fclose(fp);
	return (mac);
}

static int
ej_getclonemac(int eid, webs_t wp, int argc, char_t **argv)
{
	//char zero_mac[20];
	char str_clientip[20];
	char *str_mac;
	struct sockaddr_in peer;
	int    peer_len=sizeof(peer);
	int sockfd;
	int ret = 0;
	
	sockfd = wp->fd;
	if (getpeername(sockfd,(struct sockaddr*)&peer,&peer_len ) < 0)
	{
		return ret;
		
	}
	strcpy(str_clientip, inet_ntoa(peer.sin_addr));
	str_mac = get_http_client_hwaddr(str_clientip);
	ret = websWrite( wp, str_mac );	
	

	return ret;

}
//end of dvd.chen

#if defined(MULTI_LEVEL_MGMT)
static int
ej_get_mgmt_level(int eid, webs_t wp, int argc, char_t **argv)
{
	int ret = 0;

	ret += websWrite( wp, "%d", wp->mgmt_level );			
	return ret;

}
#endif

#if porting20081017
web_port_status_t line_status;
web_port_NeSnr_t SNR_status;
static int
ej_vdsl_main(int eid, webs_t wp, int argc, char **argv)
{
	char *name, *c;
	int ret = 0;
	
	if (ejArgs(argc, argv, "%s", &name) < 1) {
		//websError(wp, 400, "Insufficient args\n");
		return -1;
	}
	
	if(strcmp(name, "port_status")==0)
	{
		web_get_port_status(&c);
		ret += websWrite(wp, "%s", c);
	}
	
	if(strcmp(name, "line_status")==0)
	{
		memset( &line_status ,0, sizeof(web_port_status_t));
		web_get_port_status_detail(&line_status);
	}
	else if(strcmp(name, "downstream_line_rate")==0)
	{
		ret += websWrite(wp, "%s", line_status.DsLineRate);
	}
	else if(strcmp(name, "upstream_line_rate")==0)
	{
		ret += websWrite(wp, "%s", line_status.UsLineRate);
	}
#if 0  //Ares
	else if(strcmp(name, "downstream_attainable_net_rate")==0)
	{
		ret += websWrite(wp, "%s", line_status.DsAttainableNetDataRate);
	}
	else if(strcmp(name, "upstream_attainable_net_rate")==0)

	{
		ret += websWrite(wp, "%s", line_status.UsAttainableNetDataRate);
	}
#endif
	else if(strcmp(name, "transmit_power")==0)
	{
		ret += websWrite(wp, "%s", line_status.TransmitPower);
	}
	else if(strcmp(name, "farend_transmit_power")==0)
	{
		ret += websWrite(wp, "%s", line_status.FeTransmitPower);
	}

	if(strcmp(name, "get_SNR_status")==0)
	{
		memset( &SNR_status ,0, sizeof(web_port_NeSnr_t));
		web_get_port_NeSNR_Band(&SNR_status);		
	}
	else if(strcmp(name, "SNRM")==0)
	{
		ret += websWrite(wp, "%s", SNR_status.SNRM);
	}
	else if(strcmp(name, "ATTN")==0)
	{
		ret += websWrite(wp, "%s", SNR_status.ATTN);
	}
	else if(strcmp(name, "Average_SNRM")==0)
	{
		ret += websWrite(wp, "%s", SNR_status.Average_SNRM);
	}
	else if(strcmp(name, "Average_SNR")==0)
	{
		ret += websWrite(wp, "%s", SNR_status.Average_SNR);
	}
	else if(strcmp(name, "SNRM_Bands")==0)
	{
		ret += websWrite(wp, "%s", SNR_status.SNRM_Bands);
	}
	else if(strcmp(name, "LoopAttn_Bands")==0)
	{
		ret += websWrite(wp, "%s", SNR_status.LoopAttn_Bands);
	}
	else if(strcmp(name, "SATTN_Bands")==0)
	{
		ret += websWrite(wp, "%s", SNR_status.SATTN_Bands);
	}
	return ret;
}
#endif



#ifdef CONFIG_USER_INT6000

struct PIB_INFO pib_info;

static int
ej_plc_setting(int eid, webs_t wp, int argc, char **argv)
{
	char *name, *c;
	int ret = 0;
	char version[64];
	int connected_status = 1;
	char *mac_addr;
	char select[512] ;
	int i, len = 0;

	struct PLC_NODE plc_node[5] = { 0, 0, 0, 0, 0};

	memset(select, 0, sizeof(select));

	for (i=0 ; i<5 ; i++){
		strcpy(plc_node[i].mac_addr, "N/A"); 
	}

    if (ejArgs(argc, argv, "%s", &name) < 1) {
		//websError(wp, 400, "Insufficient args\n");
		return -1;
	}

	mac_addr = nvram_get("PLC_Connections_mac");
	if (strcmp(mac_addr, "N/A") == 0)   connected_status = 0;
	else {
		memset(version, 0, sizeof(version));
		intellon_setting(1, version, mac_addr);
		if (strcmp(version, "") == 0){
			connected_status = 0;
		}
		else
		{
			sprintf(select+len, "<option value=\"%s\">%s</option>",  mac_addr, mac_addr);
			len = strlen(select);	
		}	
	}


	if(strcmp(name, "connection-1") == 0)
	{
		i = 0;

		intellon_setting(0, plc_node, "ff:ff:ff:ff:ff:ff");

		while((strcmp(plc_node[i].mac_addr, "N/A") != 0) && (i<5)){

			if (strcmp(plc_node[i].mac_addr, mac_addr) != 0)
			{
				sprintf(select+len, "<option value=\"%s\">%s</option>",  plc_node[i].mac_addr,  plc_node[i].mac_addr);
				len = strlen(select);
				
			}
			i ++;
		}
		if (strcmp(select, "") == 0){
			ret += websWrite(wp, "%s", "<option value=\"N/A\">N/A</option>");
		}
		else ret += websWrite(wp, "%s", select);
	}		


    /* for VERSION INFO GET */
	if(strcmp(name, "fw_version") == 0)
	{
        if (connected_status == 0){
			ret += websWrite(wp, "%s", "N/A");
		}
		else {
			ret += websWrite(wp, "%s", version);
		}
	}
	else if(strcmp(name, "connect_status") == 0)
	{
		if (connected_status == 0){
			ret += websWrite(wp, "%s", "disconnected");
		}
		else ret += websWrite(wp, "%s", "connected");
	}

	/* for PIB INFO GET */
	else if (strcmp(name, "pib_version") == 0)
	{
		if (connected_status == 0){
			ret += websWrite(wp, "%s", "N/A");
		}
		else {
			memset(&pib_info, 0, sizeof(pib_info));
			intellon_setting(3, &pib_info, mac_addr);
			if (strcmp(pib_info.version, "") == 0)
			{
				ret += websWrite(wp, "%s", "N/A");
			}
			else ret += websWrite(wp, "%s", pib_info.version);
		}
	}
	else if (strcmp(name, "mac_addreee") == 0)
	{
		if (connected_status == 0 || (strcmp(pib_info.mac_address, "") == 0)){
			ret += websWrite(wp, "%s", "N/A");
		}
		else ret += websWrite(wp, "%s", pib_info.mac_address);
	}
	else if (strcmp(name, "DAK") == 0)
	{
		if (connected_status == 0 || (strcmp(pib_info.DAK, "") == 0)){
			ret += websWrite(wp, "%s", "N/A");
		}
		else ret += websWrite(wp, "%s", pib_info.DAK);
	}
	else if (strcmp(name, "NMK") == 0)
	{
		if (connected_status == 0 || (strcmp(pib_info.NMK, "") == 0)){
			ret += websWrite(wp, "%s", "N/A");
		}
		else ret += websWrite(wp, "%s", pib_info.NMK);
	}
	else if (strcmp(name, "network") == 0)
	{
		if (connected_status == 0 || (strcmp(pib_info.network, "") == 0)){
			ret += websWrite(wp, "%s", "N/A");
		}
		else ret += websWrite(wp, "%s", pib_info.network);
	}
	else if (strcmp(name, "MFG") == 0)
	{
		if (connected_status == 0 || (strcmp(pib_info.mfg, "") == 0)){
			ret += websWrite(wp, "%s", "N/A");
		}
		else {
			ret += websWrite(wp, "%s", pib_info.mfg);
			//nvram_set("PLC_Connections_mac", "N/A");

		}

	}
	return ret;
}


#endif


#ifdef CONFIG_USER_VOIP_ATA

static int
ej_voip_setting(int eid, webs_t wp, int argc, char **argv)
{
	char *name, *c;
	int ret = 0, i=0, len=0;
	char select[512] ;
	char value[30];
	char voip_ip[30];
	int disconnected = 0;
	
	struct voip_nodes nodes[5] = { 0, 0, 0, 0, 0};

	for (i=0 ; i<5 ; i++){
		strcpy(nodes[i].ipaddr, "N/A"); 
	}

	memset(select, 0, sizeof(select));
	memset(value, 0, sizeof(value));

    if (ejArgs(argc, argv, "%s", &name) < 1) {
		//websError(wp, 400, "Insufficient args\n");
		return -1;
	}

	strcpy(voip_ip, nvram_get("voip_nodes_ip"));
	if (strcmp(voip_ip, "N/A") == 0){
			disconnected = 1;
	}

	if(strcmp(name, "voip_nodes") == 0){
#if 1
		voip_ata_api(ATA_PROBE_DEV, (void *)nodes, NULL);

        for (i=0 ; i<5 ; i++){
			if ((strcmp(voip_ip, "N/A") != 0) && (strcmp(voip_ip, nodes[i].ipaddr) == 0)){
				sprintf(select+len, "<option value=\"%s\">%s</option>",  nodes[i].ipaddr,  nodes[i].ipaddr);
				len = strlen(select);
				break;
			}
		}

		i = 0;

		while((strcmp(nodes[i].ipaddr, "N/A") != 0) && (i<5)){

			if (strcmp(nodes[i].ipaddr, voip_ip) != 0){

				sprintf(select+len, "<option value=\"%s\">%s</option>",  nodes[i].ipaddr,  nodes[i].ipaddr);
				len = strlen(select);
            }
			i ++;
		}
#endif
		if (strcmp(select, "") == 0){
			ret += websWrite(wp, "%s", "<option value=\"N/A\">N/A</option>");
		}
		else ret += websWrite(wp, "%s", select);

	}
#if 1
	else if (strcmp(name, "voip_dev_ip") == 0)
	{
		ret += websWrite(wp, "%s", voip_ip);
	}
	else if (strcmp(name, "server") == 0)
	{
		if (disconnected == 0){
			voip_ata_api(ATA_GET_CONFIG, NULL, voip_ip);
			voip_parm_get(name, value);
			if (strcmp(value, "") != 0){
				ret += websWrite(wp, "%s", value);
			}else ret += websWrite(wp, "%s", "N/A");
		}
		else ret += websWrite(wp, "%s", "N/A");
	}
    else if (strcmp(name, "server_port") == 0)
	{
		if (disconnected == 0){
			voip_parm_get(name, value);
			if (strcmp(value, "") != 0){
				ret += websWrite(wp, "%s", value);
			}else ret += websWrite(wp, "%s", "N/A");
		}
		else ret += websWrite(wp, "%s", "N/A");
	}
	else if (strcmp(name, "port") == 0)
	{
		if (disconnected == 0){
			voip_parm_get(name, value);
			if (strcmp(value, "") != 0){
				ret += websWrite(wp, "%s", value);
			}else ret += websWrite(wp, "%s", "N/A");
		}
		else ret += websWrite(wp, "%s", "N/A");
	}
	else if (strcmp(name, "name") == 0)
	{
		if (disconnected == 0){
			voip_parm_get(name, value);
			if (strcmp(value, "") != 0){
				ret += websWrite(wp, "%s", value);
			}else ret += websWrite(wp, "%s", "N/A");
		}
		else ret += websWrite(wp, "%s", "N/A");
	}
	else if (strcmp(name, "username") == 0)
	{
		if (disconnected == 0){
			voip_parm_get(name, value);
			if (strcmp(value, "") != 0){
				ret += websWrite(wp, "%s", value);
			}else ret += websWrite(wp, "%s", "N/A");
		}
		else ret += websWrite(wp, "%s", "N/A");
	}
	else if (strcmp(name, "password") == 0)
	{
		if (disconnected == 0){
			voip_parm_get(name, value);
			if (strcmp(value, "") != 0){
				ret += websWrite(wp, "%s", value);
			}else ret += websWrite(wp, "%s", "N/A");
		}
		else ret += websWrite(wp, "%s", "N/A");
	}
	else if (strcmp(name, "Expires") == 0)
	{
		if (disconnected == 0){
			voip_parm_get(name, value);
			if (strcmp(value, "") != 0){
				ret += websWrite(wp, "%s", value);
			}else ret += websWrite(wp, "%s", "N/A");
		}
		else ret += websWrite(wp, "%s", "N/A");
	}
	else if (strcmp(name, "alive") == 0)
	{
		if (disconnected == 0){
			voip_parm_get(name, value);
			if (strcmp(value, "") != 0){
				ret += websWrite(wp, "%s", value);
			}else ret += websWrite(wp, "%s", "N/A");
		}
		else ret += websWrite(wp, "%s", "N/A");
	}
	else if (strcmp(name, "registrar") == 0)
	{
		if (disconnected == 0){
			voip_parm_get(name, value);
			if (strcmp(value, "") != 0){
				ret += websWrite(wp, "%s", value);
			}else ret += websWrite(wp, "%s", "N/A");
		}
		else ret += websWrite(wp, "%s", "N/A");
	}
	else if (strcmp(name, "stun_server") == 0)
	{
		if (disconnected == 0){
			voip_parm_get(name, value);
			if (strcmp(value, "") != 0){
				ret += websWrite(wp, "%s", value);
			}else ret += websWrite(wp, "%s", "N/A");
		}
		else ret += websWrite(wp, "%s", "N/A");
	}
	else if (strcmp(name, "rtp_port") == 0)
	{
		if (disconnected == 0){
			voip_parm_get(name, value);
			if (strcmp(value, "") != 0){
				ret += websWrite(wp, "%s", value);
			}else ret += websWrite(wp, "%s", "N/A");
		}
		else ret += websWrite(wp, "%s", "N/A");
	}
	else if (strcmp(name, "rtp_tos") == 0)
	{
		if (disconnected == 0){
			voip_parm_get(name, value);
			if (strcmp(value, "") != 0){
				ret += websWrite(wp, "%s", value);
			}else ret += websWrite(wp, "%s", "N/A");
		}
		else ret += websWrite(wp, "%s", "N/A");
	}
	else if (strcmp(name, "dtmf_pt") == 0)
	{
		if (disconnected == 0){
			voip_parm_get(name, value);
			if (strcmp(value, "") != 0){
				ret += websWrite(wp, "%s", value);
			}else ret += websWrite(wp, "%s", "N/A");
		}
		else ret += websWrite(wp, "%s", "N/A");
	}
	else if (strcmp(name, "volume") == 0)
	{
		if (disconnected == 0){
			voip_parm_get(name, value);
			if (strcmp(value, "") != 0){
				ret += websWrite(wp, "%s", value);
			}else ret += websWrite(wp, "%s", "N/A");
		}
		else ret += websWrite(wp, "%s", "N/A");
	}
	else if (strcmp(name, "gain") == 0)
	{
		if (disconnected == 0){
			voip_parm_get(name, value);
			if (strcmp(value, "") != 0){
				ret += websWrite(wp, "%s", value);
			}else ret += websWrite(wp, "%s", "N/A");
		}
		else ret += websWrite(wp, "%s", "N/A");
	}
	else if (strcmp(name, "tone") == 0)
	{
		if (disconnected == 0){
			voip_parm_get(name, value);
			if (strcmp(value, "") != 0){
				ret += websWrite(wp, "%s", value);
			}else ret += websWrite(wp, "%s", "N/A");
		}
		else ret += websWrite(wp, "%s", "N/A");
	}
	else if (strcmp(name, "reg_m") == 0)
	{
		if (disconnected == 0){
			voip_parm_get(name, value);
			if (strcmp(value, "") != 0){
				ret += websWrite(wp, "%s", value);
			}else ret += websWrite(wp, "%s", "N/A");
		}
		else ret += websWrite(wp, "%s", "N/A");
	}
	else if (strcmp(name, "tel_m") == 0)
	{
		if (disconnected == 0){
			voip_parm_get(name, value);
			if (strcmp(value, "") != 0){
				ret += websWrite(wp, "%s", value);
			}else ret += websWrite(wp, "%s", "N/A");
		}
		else ret += websWrite(wp, "%s", "N/A");
	}
#endif
	return ret;
}


#endif

/*
 * Example: 
 * lan_route=192.168.2.0:255.255.255.0:192.168.2.1:1
 * <% lan_route("ipaddr", 0); %> produces "192.168.2.0"
 */
static int
ej_lan_route(int eid, webs_t wp, int argc, char_t **argv)
{
	char *arg;
	int which;
	char word[256], *next;
	char *ipaddr, *netmask, *gateway, *metric;

	if (ejArgs(argc, argv, "%s %d", &arg, &which) < 2) {
		//websError(wp, 400, "Insufficient args\n");
		return -1;
	}

	foreach(word, nvram_safe_get("lan_route"), next) {
		if (which-- == 0) {
			netmask = word;
			ipaddr = strsep(&netmask, ":");
			if (!ipaddr || !netmask)
				continue;
			gateway = netmask;
			netmask = strsep(&gateway, ":");
			if (!netmask || !gateway)
				continue;
			metric = gateway;
			gateway = strsep(&metric, ":");
			if (!gateway || !metric)
				continue;
			if (!strcmp(arg, "ipaddr")) {
				return websWrite(wp, ipaddr);
			}	
			else if (!strcmp(arg, "netmask")) {
				return websWrite(wp, netmask);
			}	
			else if (!strcmp(arg, "gateway")) {
				return websWrite(wp, gateway);
			}	
			else if (!strcmp(arg, "metric")) {
				return websWrite(wp, metric);
			}	
		}
	}

	return 0;
}

/* Dump route in <tr><td>IP</td><td>MASK</td><td>GW</td><td>Hop Count</td><td>interface</td></tr> format */
int
ej_dump_route_table(int eid, webs_t wp, int argc, char_t **argv)
{
	int ret = 0;
	char *format;
	FILE *fp, *fp1;
	int flgs, ref, use, metric;
        unsigned long dest, gw, netmask;
	char line[256];
	int nl=0;
	struct in_addr dest_ip;
        struct in_addr gw_ip;
        struct in_addr netmask_ip;
	char sdest[16], sgw[16];
	int debug = 0, blank = 1;
	char lan_ifname[10];

	if (ejArgs(argc, argv, "%s", &format) < 1) {
		//websError(wp, 400, "Insufficient args\n");
		return -1;
	}

	strcpy(lan_ifname, nvram_safe_get("lan_ifname"));
	/* open route table*/
	if ((fp = fopen("/proc/net/route", "r")) == NULL) {
       		//websError(wp, 400, "No route table\n");
        	return -1;
        }

        /* Read the route cache entries. */
	//Iface   Destination     Gateway         Flags   RefCnt  Use     Metric  Mask            MTU     Window  IRTT                                                       
	//vmnet1  004410AC        00000000        0001    0       0       0       00FFFFFF        40      0       0                   

	while( fgets(line, sizeof(line), fp) != NULL ) {
		if (nl){
			int ifl = 0;
			while(line[ifl]!=' ' && line[ifl]!='\t' && line[ifl]!='\0')
			ifl++;
			line[ifl]=0;    /* interface */
			if(sscanf(line+ifl+1, "%lx%lx%X%d%d%d%lx", &dest, &gw, &flgs, &ref, &use, &metric, &netmask)!=7) {
				break;
		        }  
			debug = 0;
			dest_ip.s_addr = dest;
			gw_ip.s_addr   = gw;
			netmask_ip.s_addr = netmask;

			strcpy(sdest,  (dest_ip.s_addr==0 ? "0.0.0.0" : inet_ntoa(dest_ip)));    //default
			strcpy(sgw,    (gw_ip.s_addr==0   ? "0.0.0.0" : inet_ntoa(gw_ip)));      //*

		
			/* not 0x0001 route usable */
			if(!(flgs&1))	continue; 

			/* filter ppp pseudo interface for DOD */
//			if(!strcmp(sdest,PPP_PSEUDO_IP) && !strcmp(sgw,PPP_PSEUDO_GW))	debug = 1;
			
			/* Don't show loopback device */
			if(!strcmp(line,"lo"))	debug = 1;
			if(!strcmp(sdest,"0.0.0.0"))	debug = 1;

			/* Don't show eth1 information for pppoe mode */
			// Revised by wayne.lu on 3/25/2005
			// Routing table should be shown even under ppport mode
//			if(!strcmp(line, nvram_safe_get("wan_ifname")) && nvram_match("wan_proto","pppoe"))	debug = 1;

			/* Don't show pseudo interface */
/*			
			if(!strncmp(line, "ppp", 3)){
				fp1 = fopen("/tmp/ppp/link", "r");
				if(!fp1)	debug = 1;
				else		fclose(fp1);
			}
*/
			// End of revising by wayne.lu
			
			ret += websWrite(wp,"%s%c'%s','%s','%s','%s'\n",
					debug ? "//" : "",
					blank ? ' ' : ',', 
					sdest,
					inet_ntoa(netmask_ip),
					sgw,
					(strcmp(line, lan_ifname/*"br0"*/) ? "WAN" : "LAN"));    

			if(debug && blank)	blank = 1;
			else		blank = 0;


		}
		nl++;
	}
	return ret;
}

/*
 * Example: 
 * wan_route=192.168.10.0:255.255.255.0:192.168.10.1:1
 * <% wan_route("ipaddr", 0); %> produces "192.168.10.0"
 */
static int
ej_wan_route(int eid, webs_t wp, int argc, char_t **argv)
{
	char *arg;
	int which;
	char word[256], *next;
	char *ipaddr, *netmask, *gateway, *metric;
	int unit;
	char tmp[100], prefix[] = "wanXXXXXXXXXX_";


	if (ejArgs(argc, argv, "%s %d", &arg, &which) < 2) {
		//websError(wp, 400, "Insufficient args\n");
		return -1;
	}

	foreach(word, nvram_safe_get("wan_route"), next) {
		if (which-- == 0) {
			netmask = word;
			ipaddr = strsep(&netmask, ":");
			if (!ipaddr || !netmask)
				continue;
			gateway = netmask;
			netmask = strsep(&gateway, ":");
			if (!netmask || !gateway)
				continue;
			metric = gateway;
			gateway = strsep(&metric, ":");
			if (!gateway || !metric)
				continue;
			if (!strcmp(arg, "ipaddr")) {
				return websWrite(wp, ipaddr);
			}	
			else if (!strcmp(arg, "netmask")) {
				return websWrite(wp, netmask);
			}	
			else if (!strcmp(arg, "gateway")) {
				return websWrite(wp, gateway);
			}	
			else if (!strcmp(arg, "metric")) {
				return websWrite(wp, metric);
			}	
		}
	}

	return 0;
}

static int
ej_sel_bridging_get(int eid, webs_t wp, int argc, char_t **argv)
{
	char *name, *match, *output, *str, *mask, value[20];
	int type, i, j=0;

	if (ejArgs(argc, argv, "%s %s %s", &name, &match) < 2) {
		//websError(wp, 400, "Insufficient args\n");
		return -1;
	}

    type = atoi(nvram_safe_get(match));

	if (type == 2 || type == 3)
	{
		str = nvram_safe_get(name);

		mask = strchr(str, '/');

		for (i=0 ; i<strlen(mask) ; i++)
		{
			if (mask[i+1] == '0')
			{
				value[j] = '*';
				j ++;
				i ++;
			}
			else {
				if (str[i] != '/')
				{
					value[j] = str[i];
					j ++;
				}
			}
		}

		value[j] = '\0';

		return websWrite(wp, value);
	}
	else {
		str = nvram_safe_get(name);
		return websWrite(wp, str);
	}
}


static int
ej_upnp_port_list(int eid, webs_t wp, int argc, char_t **argv)
{
	FILE *fp;
	int i;
	struct in_addr addr;
	char sigusr2[] = "-XX";
	int ret = 0;
      char line[100],act[4],proto[50],ipa[50];
	int Int_port=0, Ext_port=0;
	int count = 0;	

        /* Write out upnp file */
      sprintf(line, "killall -%d upnp", SIGUSR2);
      system(line);	
      sleep(1);

	/* Parse upnp file */
	if ((fp = fopen("/var/tmp/upnp_portmap", "r")) == NULL) 
		return NULL;
	else{
			// 1       TCP     3       2       192.168.1.22
			while( fgets(line, sizeof(line), fp)!=NULL)
			{
				if(sscanf(line, "%s %s %d %d %s\n", act, proto,&Int_port, &Ext_port, ipa)!=5)
					continue;
					
				ret += websWrite(wp, "%c\"%s\",\"%s\",\"%d\",\"%d\",\"%s\"", 
					count ? ',' :' ',
					act,  
					proto,
					Int_port,
					Ext_port,
					ipa						
					);					
				count++;	

			}
	}
 	fclose(fp);
	return ret;
}

struct ej_handler ej_handlers[] = {
    { "get_firmware_ver", ej_get_firmware_ver},
	{ "nvram_get", ej_nvram_get },
	{ "nvram_match", ej_nvram_match },
	{ "nvram_invmatch", ej_nvram_invmatch },
	{ "nvram_list", ej_nvram_list },		
	{ "wanstatus_get", ej_wanstatus_get },	
	{ "localtime", ej_localtime },	
	{ "getclonemac", ej_getclonemac},	
	{ "dumplog", ej_dumplog },	
  //  { "dump_sys_log",ej_dumpSysLog},
//	{ "dumpleases", ej_dumpleases}, 
//	{ "login_ipaddr", ej_login_ipaddr },
	{ "restart_delay_get", ej_restart_delay_get },		
#if defined(MULTI_LEVEL_MGMT)
    { "mgmt_level", ej_get_mgmt_level },    
#endif	
//	{ "vdsl_main", ej_vdsl_main },
#ifdef CONFIG_USER_INT6000
	{ "plc_setting", ej_plc_setting },
#endif
#ifdef CONFIG_USER_VOIP_ATA
	{ "voip_setting", ej_voip_setting },
#endif
	{ "value_match", ej_value_match },
	{ "wan_route", ej_wan_route },
	{ "lan_route", ej_lan_route },
	{"dump_route_table",ej_dump_route_table},	
	{"sel_bridging_get",ej_sel_bridging_get},
	{"dumpupnpportmap",ej_upnp_port_list},			
	{ NULL, NULL }	
};

void
do_ej(char *path, webs_t *stream)
{
	FILE *fp = NULL;
	int c;
	char pattern[100], *asp = NULL, *func = NULL, *end = NULL;
	int len = 0;

	
	if (!(fp = fopen(path, "r")))
	{
		DPRINTF( "%s:: fopen [%s] error [%s]\n", __FUNCTION__, path, strerror( errno) );
		return;
	}

	while ((c = getc(fp)) != EOF) 
	{
		/* Add to pattern space */
		pattern[len++] = c;
		pattern[len] = '\0';
		if (len == (sizeof(pattern) - 1))
			goto release;


		/* Look for <% ... */
		if (!asp && !strncmp(pattern, "<%", len)) 
		{
			if (len == 2)
				asp = pattern + 2;
			continue;
		}

		/* Look for ... %> */
		if (asp) 
		{
			if (unqstrstr(asp, "%>")) 
			{
				for (func = asp; func < &pattern[len]; func = end) 
				{
					/* Skip initial whitespace */
					for (; isspace((int)*func); func++);
					if (!(end = unqstrstr(func, ";")))
						break;
					*end++ = '\0';

					/* Call function */
					call(func, stream);
				}
				asp = NULL;
				len = 0;
			}
			continue;
		}

release:
		/* Release pattern space */
		//websWrite(stream, pattern);		
		websWrite(stream, "%s", pattern);
		asp = NULL;
		len = 0;
	}

	if (fp != NULL) 
		fclose(fp);
}
